/* Lisp functions for i18n through GNU gettext.
   Copyright (C) 1998 Free Software Foundation, Inc.

This file is part of GNU Emacs.

GNU Emacs is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU Emacs is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU Emacs; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */


#ifdef USE_NLS /* Encloses this entire file. */

#include <locale.h>
#include <string.h>
#include "config.h"
#include "lisp.h"
#include "commands.h"
#include "buffer.h"

#include <libintl.h>
#ifndef _
#define _(s) gettext(s)
#endif

/* The Domain */
static Lisp_Object Vgettext_domain;

/* The Category */
static Lisp_Object Vgettext_category;

/* Symbols for categories */
static Lisp_Object QLC_ALL;
static Lisp_Object QLC_COLLATE;
static Lisp_Object QLC_CTYPE;
static Lisp_Object QLC_MESSAGES;
static Lisp_Object QLC_MONETARY;
static Lisp_Object QLC_NUMERIC;
static Lisp_Object QLC_RESPONSE;
static Lisp_Object QLC_TIME;

// Unsupported LC_ values are mapped to LC_MESSAGES
#if !defined(LC_ALL)
# define LC_ALL      LC_MESSAGES
#endif
#if !defined(LC_COLLATE)
# define LC_COLLATE  LC_MESSAGES
#endif
#if !defined(LC_CTYPE)
# define LC_CTYPE    LC_MESSAGES
#endif
#if !defined(LC_MONETARY)
# define LC_MONETARY LC_MESSAGES
#endif
#if !defined(LC_NUMERIC)
# define LC_NUMERIC  LC_MESSAGES
#endif
#if !defined(LC_RESPONSE)
# define LC_RESPONSE LC_MESSAGES
#endif
#if !defined(LC_TIME)
# define LC_TIME     LC_MESSAGES
#endif

static int
symbol2category(Lisp_Object symbol)
{
       if(symbol == QLC_MESSAGES) return LC_MESSAGES;
  else if(symbol == QLC_COLLATE ) return LC_COLLATE ;
  else if(symbol == QLC_CTYPE   ) return LC_CTYPE   ;
  else if(symbol == QLC_MONETARY) return LC_MONETARY;
  else if(symbol == QLC_NUMERIC ) return LC_NUMERIC ;
  else if(symbol == QLC_TIME    ) return LC_TIME    ;
  else if(symbol == QLC_RESPONSE) return LC_RESPONSE;
  else if(symbol == QLC_ALL     ) return LC_ALL     ;
  else return 0;
}

DEFUN("gettext",
      Fgettext, Sgettext, 1, 3, 0,
      "Translates STRING according to DOMAIN and CATEGORY.\n
If DOMAIN and/or DOMAIN are nil or unspecified, the buffer's default values\n\
 (as specified by `gettext-domain' and `gettext-category') are used instead.\n\
If no language is set (in $LANG or $LANGUAGE), or if no translation was found,\n\
 STRING is returned unchanged.\n\
You can control where this function finds its message catalogs by calling\n\
 `bind-gettext-domain'.\n")
     (string, domain, category)
     Lisp_Object string, domain, category;
{
Lisp_Object Lmessage = Qnil;
int   cat = 0;
char* msg = NULL;
  if(NILP(string)) return Qnil;
  CHECK_STRING(string, 0);
  if(NILP(domain)) domain     = Vgettext_domain;
  CHECK_STRING(domain, 0);
  if(NILP(category)) category = Vgettext_category;
  CHECK_SYMBOL(category, 0);
  cat = symbol2category(category);
  if(cat == 0) {
    error("Unknown or unsupported message category (%s)", category);
    return Qnil;
  }
  msg = dcgettext(XSTRING(domain)->data, XSTRING(string)->data, cat);
  if(msg) Lmessage = build_string(msg);
  return  Lmessage;
}

DEFUN("bind-gettext-domain",
      Fbindtextdomain, Sbindtextdomain, 2, 2, 0,
      "Sets the message directory for DOMAIN to DIRECTORY.\n\
If t, messages for DOMAIN will be subsequently retrieved from a subdirectory\n\
 'LANGUAGE/CATEGORY' of DIRECTORY.\n\
If nil, DOMAIN couldn't be bound to DIRECTORY.\n")
     (domain, directory)
     Lisp_Object domain, directory;
{
Lisp_Object Lold = Qnil;
char* s = NULL;
  CHECK_STRING(domain, 0);
  CHECK_STRING(directory, 0);
  if(NILP(domain   )) return Qnil;
  if(NILP(directory)) return Qnil;
  s = bindtextdomain(XSTRING(domain)->data, XSTRING(directory)->data);
  if(!s) return Qnil;
  Lold = build_string(s);
  return Qt;
}

/* module initialization */
void
syms_of_gettext()
{
  QLC_ALL      = intern("LC_ALL"     );
  QLC_COLLATE  = intern("LC_COLLATE" );
  QLC_CTYPE    = intern("LC_CTYPE"   );
  QLC_MESSAGES = intern("LC_MESSAGES");
  QLC_MONETARY = intern("LC_MONETARY");
  QLC_NUMERIC  = intern("LC_NUMERIC" );
  QLC_RESPONSE = intern("LC_RESPONSE");
  QLC_TIME     = intern("LC_TIME"    );
  staticpro(&QLC_ALL     );
  staticpro(&QLC_COLLATE );
  staticpro(&QLC_CTYPE   );
  staticpro(&QLC_MESSAGES);
  staticpro(&QLC_MONETARY);
  staticpro(&QLC_NUMERIC );
  staticpro(&QLC_RESPONSE);
  staticpro(&QLC_TIME    );

  Vgettext_category = QLC_MESSAGES;
  Vgettext_domain   = build_string("emacs");
  staticpro(&Vgettext_category);
  staticpro(&Vgettext_domain  );

  // Sets up translation for internal use by emacs
  textdomain("emacs");

  defsubr(&Sgettext);
  defsubr(&Sbindtextdomain);

  /* These should really be buffer-local, but my attempts to add slots for
     them to the buffer structure almays led Emacs to crash & burn. */
  DEFVAR_LISP("gettext-domain", &Vgettext_domain,
	      "The message domain used by `gettext'.\n\
Any string is a valid value; see the documentation for `gettext' to see\n\
 how this affects message translation.\n");

  DEFVAR_LISP("gettext-category", &Vgettext_category,
	      "The message category used by `gettext'.\n\
All locale categories are supported as symbols of the same name. They are:\n\
  'LC_MESSAGES (the default)  'LC_COLLATE  'LC_CTYPE  'LC_RESPONSE
  'LC_MONETARY                'LC_NUMERIC  'LC_TIME   'LC_ALL
Note that LC_* constants that aren't supported by a system, are mapped to
 LC_MESSAGES instead.
For how this affects message translation, see the documentation for `gettext'.\
\n");
}

#else

#error I am for USE_NLS use only.

#endif
